/*   This file is part of Xsltop.
*
*    Xsltop is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation.
*
*    Xsltop is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY.  See the
*    GNU General Public License for more details.
*
*    See <http://www.gnu.org/licenses/>.
*/
package xsltop.xml;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;


/**
 * The Class XmlBaseEngine.
 */
public class XmlBaseEngine {
	
	/** The document. */
	private Document data = null;
	
	/** The namespaceaware. */
	private boolean namespaceaware = true;
	
	/**
	 * Instantiates a new xml base engine.
	 */
	@SuppressWarnings("unused")
	private XmlBaseEngine() {
		super();
	}
	
	/**
	 * Instantiates a new xml base engine.
	 * 
	 * @param stream the stream
	 * @param validate the validate
	 * @param namespaceaware the namespaceaware
	 * 
	 * @throws XsltopException the xsltop exception
	 */
	public XmlBaseEngine(final InputStream stream, boolean validate, boolean namespaceaware) throws XsltopException {
		setNamespaceaware(namespaceaware);
		data = loadStream(stream, validate);
	}	
	
	/**
	 * Sets the value.
	 * 
	 * @param pXpath the xpath
	 * @param pValue the value
	 * 
	 * @throws XsltopException the xsltop exception
	 */
	public void setValue(String pXpath, String pValue) throws XsltopException {
		Node n = this.evaluateXPathSingle(pXpath);
		n.setNodeValue(pValue);
	}
	
	/**
	 * Load stream.
	 * 
	 * @param pStream the stream
	 * @param pValidate the validate
	 * 
	 * @return the document
	 * 
	 * @throws XsltopException the xsltop exception
	 */
	public Document loadStream(final InputStream pStream,
			final boolean pValidate) throws XsltopException  {
		Document doc = null;
		try {			
            final DocumentBuilder vBuilder = getDocumentBuilder(pValidate);
            vBuilder.setErrorHandler(new XsltopHandler());
            vBuilder.setEntityResolver(new XsltopFinder());            
            doc = vBuilder.parse(pStream);
		} catch (ParserConfigurationException e) {
			data = null;
			throw new XsltopException(e);
		} catch (IOException e) {
			data = null;
			throw new XsltopException(e);
		} catch (SAXException e) {
			data = null;
			throw new XsltopException(e);
		} finally {
			try {
				pStream.close();
			} catch (IOException e) {
				System.out.println("stream.close() " + e.getMessage());
			}
		}
		return doc;
	}
	
	/**
	 * Gets the document builder.
	 * 
	 * @param pValidate the validate
	 * 
	 * @return the document builder
	 * 
	 * @throws ParserConfigurationException the parser configuration exception
	 */
	protected DocumentBuilder getDocumentBuilder(final boolean pValidate) 
		throws ParserConfigurationException {
		final DocumentBuilderFactory factory = 
        								DocumentBuilderFactory.newInstance();
        factory.setValidating(pValidate);
        factory.setNamespaceAware(this.namespaceaware);
        return factory.newDocumentBuilder();
	}
	
	/**
	 * Evaluate x path.
	 * 
	 * @param pExpression the expression
	 * 
	 * @return the list< string>
	 * 
	 * @throws XsltopException the xsltop exception
	 */
	public List<String> evaluateXPath(final String pExpression) throws XsltopException {		
		NodeList nodelist = XPathHelper.evalList(data, pExpression);
		List<String> expressions = new ArrayList<String>();
		for(int index=0;index<nodelist.getLength();++index) {
			Node node = nodelist.item(index);
			String texte = node.getTextContent().trim().replaceAll("  ", "");
			texte = texte.replaceAll("\n", "").replaceAll("\t", "");
			expressions.add("[name=" + node.getNodeName()
					+ ", value=" + node.getNodeValue()
					+ ", type=" + node.getNodeType() + "]");
		}
		return expressions;
	}
	
	/**
	 * Evaluate x path list.
	 * 
	 * @param pExpression the expression
	 * 
	 * @return the list< string>
	 * 
	 * @throws XsltopException the xsltop exception
	 */
	public List<String> evaluateXPathList(final String pExpression) throws XsltopException {		
		NodeList nodelist = XPathHelper.evalList(data, pExpression);
		List<String> expressions = new ArrayList<String>();
		for(int index = 0; index < nodelist.getLength(); index++) {
			Node node = nodelist.item(index);
			String texte = node.getTextContent().trim().replaceAll("  ", "");
			texte = texte.replaceAll("\n", "").replaceAll("\t", "");
			expressions.add(node.getNodeValue());
		}
		return expressions;
	}
	
	/**
	 * Evaluate x path single.
	 * 
	 * @param pExpression the expression
	 * 
	 * @return the node
	 * 
	 * @throws XsltopException the xsltop exception
	 */
	public Node evaluateXPathSingle(final String pExpression)
	throws XsltopException {
		return XPathHelper.eval(data, pExpression);
	}
	
	/**
	 * Gets the document.
	 * 
	 * @return the document
	 */
	public Document getDocument() {
		return data;
	}
	
	/**
	 * Xml to string.
	 * 
	 * @return the string
	 * 
	 * @throws XsltopException the xsltop exception
	 */
	public String exportToString() throws XsltopException {
		return XsltopTransformer.exportToString(data);
	}
	
	/**
	 * Xml to file.
	 * 
	 * @param pFile the file
	 * 
	 * @throws XsltopException the xsltop exception
	 */
	public void exportToFile(final File pFile) throws XsltopException {
		XsltopTransformer.exportToFile(data, pFile);
	}
	
	/**
	 * Sets the namespaceaware.
	 * 
	 * @param pNamespaceaware the new namespaceaware
	 */
	public void setNamespaceaware(boolean pNamespaceaware) {
		this.namespaceaware  = pNamespaceaware;
	}
}
